use std::{collections::BTreeMap, sync::Arc};

use vulkano::{
    Validated, VulkanError,
    descriptor_set::{
        DescriptorSet,
        allocator::StandardDescriptorSetAllocator,
        layout::{DescriptorSetLayout, DescriptorSetLayoutBinding},
    },
};

mod buffer;
mod command;

pub mod camera;
pub mod mvp;
pub mod resource;
pub mod transform;
pub mod vertex;
pub use buffer::{AsBindableBuffer, AsIndexBuffer, AsUniformBuffer, AsVertexBuffer};
pub use command::{AsRecordable, AsRenderableMesh, AsRenderableMeshInstance};

pub trait AsBindableDescriptorSet<T> {
    fn as_descriptor_set_layout_bindings() -> BTreeMap<u32, DescriptorSetLayoutBinding>;

    fn as_descriptor_set(
        descriptor_set_allocator: &Arc<StandardDescriptorSetAllocator>,
        layout: &Arc<DescriptorSetLayout>,
        data: &T,
    ) -> Result<Arc<DescriptorSet>, Validated<VulkanError>>;
}
